// LED Table, Written by Theron Wierenga, September 2016

#include <Adafruit_NeoPixel.h>
#include <avr/power.h>
#include <Wire.h>
#include "RTClib.h"

#define NUMPIXELS  300  // Change for number of LEDs
#define NEOPIN 6
#define HOUR 0
#define MIN 1
#define BLACK 0
#define RED 1
#define ORANGE 2
#define YELLOW 3
#define GREEN 4
#define CYAN 5
#define BLUE 6
#define VIOLET 7

Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUMPIXELS, NEOPIN, NEO_GRB + NEO_KHZ800);
RTC_DS3231 rtc;

#define analogPinRight A0  // read from multiplexer using analog input 0
#define analogPinLeft A1   // read from multiplexer using analog input 1
#define strobePinRight 3   // strobe is attached to digital pin 3
#define resetPinRight 2    // reset is attached to digital pin 2
#define strobePinLeft 5    // strobe is attached to digital pin 5
#define resetPinLeft 4     // reset is attached to digital pin 4
int spectrumValueRight[7]; // to hold a2d values
int spectrumValueLeft[7];  // to hold a2d values
// Create an array of 8 standard colors
// black, red, orange, yellow, green, cyan, blue, violet
uint32_t colors[8] = { pixels.Color(0, 0, 0), pixels.Color(255, 0, 0),
                       pixels.Color(255, 60, 0), pixels.Color(255, 255, 0),
                       pixels.Color(0, 255, 0), pixels.Color(0, 255, 255), 
                       pixels.Color(0, 0, 255), pixels.Color(148, 0, 211) };
uint32_t color;
// 5x7 font character table for numbers
byte Font1[10][7] = { 
   {0x0e, 0x11, 0x11, 0x11, 0x11, 0x11, 0x0e},   // 0 
   {0x04, 0x0c, 0x04, 0x04, 0x04, 0x04, 0x0e},   // 1 
   {0x0e, 0x11, 0x01, 0x02, 0x04, 0x08, 0x1f},   // 2 
   {0x0e, 0x11, 0x01, 0x06, 0x01, 0x11, 0x0e},   // 3 
   {0x02, 0x06, 0x0a, 0x12, 0x1f, 0x02, 0x02},   // 4 
   {0x1f, 0x10, 0x1e, 0x01, 0x01, 0x11, 0x0e},   // 5 
   {0x06, 0x08, 0x10, 0x1e, 0x11, 0x11, 0x0e},   // 6 
   {0x1f, 0x01, 0x02, 0x04, 0x08, 0x08, 0x08},   // 7 
   {0x0e, 0x11, 0x11, 0x0e, 0x11, 0x11, 0x0e},   // 8 
   {0x0e, 0x11, 0x11, 0x0f, 0x01, 0x02, 0x0c}    // 9 
};

void setup() {
  
  Serial.begin(115200);
  pixels.begin();               // This initializes the NeoPixel library.
  pixels.setBrightness(32);     // Brightness can be set from 0 - 255.

  // Set the strobe and reset pins as outputs
  pinMode(strobePinRight, OUTPUT);
  pinMode(resetPinRight, OUTPUT);
  pinMode(strobePinLeft, OUTPUT);
  pinMode(resetPinLeft, OUTPUT);
  // Set pins 7, 8 and 9 as INPUT_PULLUPs
  pinMode(7, INPUT_PULLUP);
  pinMode(8, INPUT_PULLUP);
  pinMode(9, INPUT_PULLUP);
  // Read analog values from 0 - 5 volts
  analogReference(DEFAULT);
  // Set strobe and reset pins to initial values
  digitalWrite(resetPinRight, LOW);
  digitalWrite(strobePinRight, HIGH);
  digitalWrite(resetPinLeft, LOW);
  digitalWrite(strobePinLeft, HIGH);
  // Start the Real Time Clock
  rtc.begin();
  // Set the Real Time clock to the current time desired
  // ****************************************************
  // SEE SIDEBAR FOR DETAILS ON COMMENTING OUT THIS LINE
  // AFTER SETTING THE TIME AND RECOMPILING AND UPLOADING
  // ****************************************************
  // year, month, day, hour minute, second
  rtc.adjust(DateTime(2016, 9, 7, 18, 26, 1));
}

void loop() {
 int function = digitalRead(9) * 4 + digitalRead(8) * 2 + digitalRead(7);
 switch (function)
 {
    case 0: freq();                 // Display Graphic Equilizer
            break;
    case 1: getTime(MIN);           // Display Clock with minutes, seconds
            break;
    case 2: getTime(HOUR);          // Display Clock with hours, minutes
            break;
    case 3: exercise1();            // Display color sweeps across array
            break;
    case 4: colorcube();            // Display 6 faces of color cube
            break;
    case 5: // Do nothing, for expansion
            break;
    case 6: // Do nothing, for expansion
            break;
    case 7: // Do nothing, for expansion
            break;
 }
}

// This functions cycles through 64 different colors
void exercise1()
{
  int num, xx, yy;
  for (int r = 0; r < 256; r+=64)           // red
  { 
    for (int g = 0; g < 256; g+=64)         // green
    {
      for (int b = 0; b < 256; b+=64)       // blue
      {
        for (int dir = 0; dir < 2; dir++)   // two directions
        { 
          for (int z = 0; z < 2; z++)       // sweep on then off
          {
            for (int x = 0; x < 15; x++)    // width
            {
              for (int y = 0; y < 10; y++)  // height
              {
                switch (dir)
                {
                  case 0: yy = y; xx = x;   // to the right
                          break;
                  case 1: yy = 9 - y; xx = 14 - x;    // to the left
                          break;
                }
                num = yy * 30 + xx * 2;
                switch (z)
                {
                  case 0:pixels.setPixelColor(num, r, g, b); 
                         pixels.setPixelColor(num + 1, r, g, b); 
                         break;
                  case 1:pixels.setPixelColor(num, colors[BLACK]); 
                         pixels.setPixelColor(num + 1, colors[BLACK]); 
                         break;
                }
              }
              pixels.show();
              delay(20);
            }   
          }
        }
      }
    }
  }
}

// Display the six faces of a color cube
void colorcube()
{
  int r,g,b,num;
  r = 255; g = 252;
  for(int y = 0; y < 5; y++)
  {
    b = 0;
    for(int x = 0; x < 5; x++)
    {
      num = y * 30 + x * 2;
      pixels.setPixelColor(num, r, g, b);
      pixels.setPixelColor(num + 1, r, g, b);
       b = b + 63;
    }
    g = g - 63;
  }
  pixels.show();
  r = 252; b = 255;
  for(int y = 0; y < 5; y++)
  {
    g = 0;
    for(int x = 5; x < 10; x++)
    {
      num = y * 30 + x * 2;
      pixels.setPixelColor(num, r, g, b);
      pixels.setPixelColor(num + 1, r, g, b);
      g = g + 63;
    }
    r = r - 63;
  }
  pixels.show();
  g = 255; b = 252;
  for(int y = 0; y < 5; y++)
  {
    r = 0;
    for(int x = 10; x < 15; x++)
    {
      num = y * 30 + x * 2;
      pixels.setPixelColor(num, r, g, b);
      pixels.setPixelColor(num + 1, r, g, b);
      r = r + 63;
    }
    b = b - 63;
  }
  pixels.show();

  g = 0; b = 0;
  for(int y = 5; y < 10; y++)
  {
    r = 0;
    for(int x = 0; x < 5; x++)
    {
      num = y * 30 + x * 2;
      pixels.setPixelColor(num, r, g, b);
      pixels.setPixelColor(num + 1, r, g, b);
      r = r + 63;
    }
    b = b + 63;
  }
  pixels.show();
  r = 0; b = 0;
  for(int y = 5; y < 10; y++)
  {
    g = 0;
    for(int x = 5; x < 10; x++)
    {
      num = y * 30 + x * 2;
      pixels.setPixelColor(num, r, g, b);
      pixels.setPixelColor(num + 1, r, g, b);
      g = g + 63;
    }
    b = b + 63;
  }
  pixels.show();
  r = 0; b = 0;
  for(int y = 5; y < 10; y++)
  {
    g = 0;
    for(int x = 10; x < 15; x++)
    {
      num = y * 30 + x * 2;
      pixels.setPixelColor(num, r, g, b);
      pixels.setPixelColor(num + 1, r, g, b);
      g = g + 63;
    }
     r = r + 63;
  }
  pixels.show();
}

// Read and display the time
// hourmin set to HOUR for a 4 digit hour and minute clock
// or hourmin set to MIN fpr a 4 digit minute and second clock 
void getTime(int hourmin)
{
  // Read the current time from DS3231 RTC 
   DateTime now = rtc.now();
   int h = now.hour();
   if (h > 12) h = h - 12;  // a 12 hour clock, remove for 24 hour
   int m = now.minute();
   int s = now.second();
   // now split in 10s and 1s digits
   int h10 = h / 10;
   int h1 = h % 10;
   int m10 = m / 10;
   int m1 = m % 10;
   int s10 = s / 10;
   int s1 = s % 10;

   // Now display the 4 digits
   switch (hourmin)
   {
    case HOUR: setLedNumbers(h10, 0);
               setLedNumbers(h1, 1);
               setLedNumbers(m10, 2);
               setLedNumbers(m1, 3);
               break;
    case MIN:  setLedNumbers(m10, 0);
               setLedNumbers(m1, 1);
               setLedNumbers(s10, 2);
               setLedNumbers(s1, 3);
               break;
   }
}

// Display a single digit
// place = 0 displays digit in UL
// place = 1 displays digit in UR
// place = 2 displays digit in LL
// place = 3 displays digit in LR
void setLedNumbers(int digit, int place)
{
  int num;
  byte p, q;

  // Set the color of each digit
  switch (place)
  {
    case 0: color = colors[ORANGE];
            break;
    case 1: color = colors[ORANGE];
            break;
    case 2: color = colors[BLUE];
            break;
    case 3: color = colors[BLUE];
            break;
  }

  for (int y = 0; y < 7; y++)     // seven rows
  {
      p = 16;
      for(int x = 0; x < 5; x++)  // five columns
      {
        q = Font1[digit][y] & p;  // AND ech bit in the 5x7 array
        switch (place)
        {
          case 0: // num = x * 30 + y * 2;              // These alternate values just
                                                        // switches display horizontally
                  num = (9 - x) * 30 + (28 - (y * 2));
                  break;
          case 1: // num = (x + 5) * 30 + y * 2;
                  num = (4 - x) * 30 + (28 - (y * 2));
                  break;
          case 2: // num = x * 30 + y * 2 + 16;
                  num = (9 - x) * 30 + (12 - (y * 2));
                  break;
          case 3: // num = (x + 5) * 30 + y * 2 + 16;
                  num = (4 - x) * 30 + (12 - (y * 2));
                  break;
        }
        // Does the value in the 5x7 array need to be turned on?
        if (q > 0)
        {
          pixels.setPixelColor(num, color); 
          pixels.setPixelColor(num + 1, color);
          Serial.print("*"); 
        }
        else
        {
          pixels.setPixelColor(num, colors[BLACK]);
          pixels.setPixelColor(num + 1, colors[BLACK]);
          Serial.print(" "); 
        }
        // Shift once to right for next bit
        p = p >> 1;
      }
   }
   // Now turn on the display
   pixels.show();
}

// Read the two stereo audio channels and display
// a graphic equalizer
void freq()
{
  int num;
  // Read frequencies and display
  // Left channel
  // Do a reset
  digitalWrite(resetPinLeft, HIGH);
  digitalWrite(resetPinLeft, LOW);

  // Read the left seven frequency channels
  for (int i = 0; i < 7; i++)
  {
     // Set stobe low then read a value
     digitalWrite(strobePinLeft, LOW);
     // Allow output to settle
     delayMicroseconds(30);
     // Diving by 103 reduces the 0 - 1023 output to 0 to 9
     spectrumValueLeft[i] = analogRead(analogPinLeft) / 103;
     // Return strobe to high, ready for another read
     digitalWrite(strobePinLeft, HIGH);
  }
 
  // Right channel
  digitalWrite(resetPinRight, HIGH);
  digitalWrite(resetPinRight, LOW);

  // Read the right seven frequency channels
  for (int i = 0; i < 7; i++)
  {
     digitalWrite(strobePinRight, LOW);
     delayMicroseconds(30);
     spectrumValueRight[i] = analogRead(analogPinRight) / 103;
     digitalWrite(strobePinRight, HIGH);
   //  Serial.print(spectrumValueRight[i]);
  }

// Now display the results
// Left channel
  for (int x = 0; x < 7; x++)             // seven channels
  {
     for (int y = 0; y < 10; y++)         // 10 different heights 
     {                                    // repesenting frequency strength
        num = y * 30 + x * 2;         
        if (spectrumValueLeft[x] >= y)    // add a color to the bar height
        {
            pixels.setPixelColor(num, colors[x + 1]);
            pixels.setPixelColor(num + 1, colors[x + 1]);
        }
         else
        {                                 // or fill in the top with no color
            pixels.setPixelColor(num, 0, 0, 0);
            pixels.setPixelColor(num + 1, 0, 0, 0);
        }
     }
  }
  
// Right channel
  for (int x = 0; x < 7; x++)
  {
     for (int y = 0; y < 10; y++)
     {
        num = y * 30 + x * 2 + 16;
        if (spectrumValueRight[x] >= y)
        {
            pixels.setPixelColor(num, colors[x + 1]);
            pixels.setPixelColor(num + 1, colors[x + 1]);
        }
        else
        {
            pixels.setPixelColor(num, 0, 0, 0);
            pixels.setPixelColor(num + 1, 0, 0, 0);
        }
     }
  }
  // after every LED is programmed turn them on
  pixels.show(); 
}

